
;; This module was coded on 12 June 2002 by Arthur O'Dwyer,
;; for use in the Day-9 Language Compiler, version one.dot.beta.
;; When assembled, this module masses 122 bytes.
;;
;; All code related to the Day-9 Compiler is freely redistributable.



    ; Get arguments from command line and push them onto the stack:
    ; #0 is name of executing program; then
    ; #1 will be argument 1, #2 argument 2, and so on.
    ; The arguments end with a null pointer.  No equivalent of argc.


    ; Now we have to parse a string of arguments.
    ; It looks like this:
    ; (ws) ARGUMENT (ws) "ARG (ws) UMENT" (ws) ... [carriage return]
    ; We will treat "quotes" *and* 'chotes' as argument delimiters.
    ; Thus we need two flags: 'CX' and "DX".
    ; We simply throw the arguments on top of the stack, with convenient
    ; NULs between them, and pushing the stack down as we go.

    MOV   AL, BYTE PTR [$80]            ; number of characters in args

    SHL   AX, 1                         ; AL holds length of arg string
    SUB   SP, AX                        ; first, push down the stack

    AND   SP, -2                        ; align SP on word boundary
                                        ; (-2 == $FFFE)

    MOV   DI, SP                        ; DI holds new location of args

    MOV   BX, $80                       ; BX holds old location of args
    XOR   CX, CX                        ; CX is 'in-chotes'
    XOR   DX, DX                        ; DX is "in-quotes"

    XOR   AH, AH                        ; keep AH zeroed

@argwsloop:                             ; clean initial wspace characters
    INC   BX
    MOV   AL, BYTE PTR [BX]
    CMP   AL, 32
    JE    @argwsloop

    DEC   BX                            ; silly, yeah...

@argloop:
    INC   BX                            ; advance the "old arg" pointer
    MOV   AL, BYTE PTR [BX]             ; AL holds current character
    CMP   AL, 34
    JNE   @argnqu
    OR    CX, CX                        ; quotes don't work in chotes
    JNZ   @argnqu
    XOR   DL, 1                         ; toggle "in-quotes"
    JMP   SHORT @argloop
@argnqu:                                ; not a "quote" ...
    CMP   AL, 39
    JNE   @argnch
    OR    DX, DX                        ; chotes don't work in quotes
    JNZ   @argnch
    XOR   CL, 1                         ; toggle 'in-chotes'
    JMP   SHORT @argloop
@argnch:                                ; not a 'chote' ...
                                        ; just a regular character
    CMP   AL, 13                        ; end of args?
    JE    @argsdone                     ; args ended; finish off

    CMP   AL, 32                        ; whitespace?
    JNE   @argchar                      ; no, add it to current arg

    MOV   SI, CX                        ; inside ch/qu/otes, wspace is OK
    OR    SI, DX                        ; SI is temp. variable here
    JNZ   @argchar
                                        ; else it's a new argument
    MOV   AL, 0                         ; end of argument
    STOSW                               ; AL=AX=0; set [DI] to 0 too.

    JMP   SHORT @argwsloop              ; clean extra wspace + get next char

@argchar:
    STOSW                               ; store it and advance DI by 2
    JMP   SHORT @argloop

@argsdone:

    XOR   AX, AX
    CMP   WORD PTR [DI+-2], AX          ; did we end with wspace?
    JNE   @argspcase                    ; don't double-NUL the string!
    DEC   DI
    DEC   DI
@argspcase:
    MOV   WORD PTR [DI], AX             ; end of argument

    MOV   BX, SP                        ; store old SP here
    CMP   DI, BX                        ; if DI has moved, we have args
                                        ;  |
    PUSH  AX                            ; push end of argument list
                                        ;  |
                                        ;  V
    JZ    @startdone                    ; condition drops to here; don't
                                        ; push the first arg, just exit.

    ; Now, push the arguments onto the stack!
    ; This could get tricky.
    ; Starting at the top (where DI is now), and going down to where SP
    ; is now (temp. stored in BX), after encountering every 0000 word,
    ; push that address/2 + 1 onto the stack.

@argpushloop:
    DEC   DI
    DEC   DI
    CMP   DI, BX
    JL    @argpushend
    MOV   AX, WORD PTR [DI]
    OR    AX, AX
    JNZ   @argpushloop
    MOV   AX, DI
    SHR   AX, 1
    INC   AX
    PUSH  AX
    JMP   SHORT @argpushloop
@argpushend:

    SHR   BX, 1
    PUSH  BX                            ; push the first arg

@startdone:

